from rest_framework import serializers
from django.contrib.auth import get_user_model

from chat_app.models import ChatRoom, GroupMessage, GroupReport, MediaFile
User = get_user_model()

class MessageSendSerializer(serializers.Serializer):
    receiver_id = serializers.IntegerField()
    message = serializers.CharField(required=False, allow_blank=True)
    images = serializers.ListField(
        child=serializers.ImageField(),
        required=False,
        allow_empty=True
    )
    video = serializers.FileField(required=False, allow_null=True)
    file = serializers.FileField(required=False, allow_null=True)
    voice = serializers.FileField(required=False, allow_null=True)
    def validate(self, data):
        message = data.get('message', '').strip()
        images = data.get('images', [])
        video = data.get('video')
        file = data.get('file')
        voice = data.get('voice')
        if not message and not images and not video and not file and not voice:
            raise serializers.ValidationError("Either a message, images, audio, video, file or voice must be provided.")
        return data

    def validate_receiver_id(self, value):
        if not User.objects.filter(id=value).exists():
            raise serializers.ValidationError("Receiver not found.")
        return value

class CreateGroupSerializer(serializers.Serializer):
    name = serializers.CharField(max_length=255)
    description = serializers.CharField(required=False, allow_blank=True)
    # members = serializers.ListField(
    #     child=serializers.IntegerField(),
    #     required=True,
    #     allow_empty=False,
    # )
    profile_image = serializers.ImageField(required=False)

    # def validate_members(self, value):
    #     if not value:
    #         raise serializers.ValidationError("At least one member must be provided.")
    #     return value
    
class SendGroupMessageSerializer(serializers.Serializer):
    group_id = serializers.IntegerField()
    content = serializers.CharField(required=False, allow_blank=True)
    images = serializers.ListField(
        child=serializers.ImageField(),
        required=False,
        allow_empty=True,
    )
    video = serializers.FileField(required=False, allow_null=True)
    file = serializers.FileField(required=False, allow_null=True)
    voice = serializers.FileField(required=False, allow_null=True)

    def validate(self, data):
        # Ensure at least one of content, images, or video is provided
        if not data.get('content') and not data.get('images') and not data.get('video') and not data.get('file') and not data.get('voice'):
            raise serializers.ValidationError(
                "You must provide either content, images, a video, file or voice message"
            )
        return data

class ChatRoomSerializer(serializers.ModelSerializer):
    owner = serializers.StringRelatedField()
    members = serializers.StringRelatedField(many=True)

    class Meta:
        model = ChatRoom
        fields = ["conversation_id", "name", "description", "owner", "members", "max_users", "created_at"]

# class GroupReportSerializer(serializers.ModelSerializer):
#     class Meta:
#         model = GroupReport
#         fields = ["id", "reporter", "group", "created_at"]
#         read_only_fields = ["reporter", "group", "created_at"]

#     def create(self, validated_data):
#         # Get group from context and add to validated data
#         validated_data["group"] = self.context["group"]
#         return super().create(validated_data)
